// mCookie Game Console Main.
// Develop new games by publicly inheriting from the Game class.

// Note: The LED Matrix is rotated 90 degrees clockwise to fit correctly in the LEGO block design.
// This means that (00,00) is the upper right hand corner
//                 (15,15) is the bottom left
//                 (15,00) is the bottom right
//                 (00,15) is the upper left
#include <Wire.h>
#include <IRremote.h> //Import the library for the Infrared Receiver.
#include "IRControl.h"
#include "userDef.h"
#include "Microduino_Matrix.h"
#include "Snake.h"
#include <avr/interrupt.h>

// Function Declaration.
void ISR_display();
void menu(uint32_t);

uint8_t Addr[MatrixPix_X][MatrixPix_Y] = {  //2x2
  { 64, 63},
  { 62, 61}
};

// Global Object Declaration
Matrix display = Matrix(Addr);  // Pixel Matrix for the display.
IRrecv irrecv(PIN_RECV);        // IR Reciever.
decode_results results;         // IR Reciever data carrier.
Game* game = nullptr;                     // Game Pointer.

uint8_t mode = MODE_MENU;

// **************** BEGINNING OF SETUP ****************** //
void setup() {

  Wire.begin();
  irrecv.enableIRIn();//Initialize the Infrared Receiver and begin receiving data.

  // Drawing four dots marking the edges of the screen.
  display.clearDisplay();
  display.setLedColorFast(BOARD_HEIGHT-1, BOARD_WIDTH-1, 255,000,000);
  display.setLedColorFast(            00, BOARD_WIDTH-1, 000,255,000);
  display.setLedColorFast(BOARD_HEIGHT-1,            00, 000,000,255);
  display.setLedColorFast(            00,            00, 255,255,255);


// Serial debugging in setup.
#if DEBUG
  Serial.begin(115200); // See the connection status in Serial Monitor
  Serial.println("**************START************");
  //getDeviceAddr
  for (int a = 0; a < display.getMatrixNum(); a++) {
    Serial.print(display.getDeviceAddr(a));
    Serial.print(" ");
  }
#endif
  
}
// ******************* END OF SETUP ********************* //

// **************** BEGINNING OF LOOP ******************* //
void loop() { 
  
  if (irrecv.decode(&results)) //If there is an available Infrared signal, then:
  {
#if DEBUG
    Serial.print("Recv Code:");
    Serial.println(results.value, HEX);
#endif

    // Checks to see whether a game is selected or not.
    if (game == nullptr)
      menu(results.value);
    else
    {     
      // Toggles play state.
      if (BUTTON_OK == results.value)
        mode = !mode;
  
      // Send the remote command into the game if the game is active.
      if (mode)
        game->parseIR(results.value);
      else
        menu(results.value);
      //irrecv.enableIRIn();
    }
    irrecv.resume(); //Receive next value.
  }

  // Runs the game's next step if there is an active game.
  if (mode && game != nullptr)
  {
    game->playGame();
    updateDisplay();
  }
   
  delay(5);
}
// ******************* END OF LOOP ********************** //

// Dispaly updates from history of the currently selected game.
void updateDisplay()
{
#if DEBUG
  Serial.println("Running");
#endif
  display.clearDisplay();
  uint8_t pos = game->getHistory(0), numItems = game->getNumItems();
  // Cycle through each item in the game and draw it to the screen.
  display.setLedColorFast(pos/BOARD_HEIGHT, pos%BOARD_HEIGHT, 0,255,0);
  for (int i = 1; i < numItems; i++){
    pos = game->getHistory(i);
    display.setLedColorFast(pos/BOARD_HEIGHT, pos%BOARD_HEIGHT, 255,0,0);
  }
}

// The menu assigns a game to the game object based on the user selection.
void menu (uint32_t recvCode)
{
  switch(recvCode)
  {
     case BUTTON_A:
        delete game;
        game = new GAME_A;
        break;
     default:
        break;
  }
  recvCode = 0;
}

