// Function implementations of the Snake Game.

#include "Snake.h"
#include "IRControl.h"
#include <Arduino.h>

// Constructor.
Snake::Snake ()
{
  newGame();
}

// Checks to see if the location selected intercepts the snake.
bool Snake::isOpen(int pos) {
  for (int i = numItems; i > 2; i--)
    if (pos == history[i])
      return false;
  return true;
}

// Generates a new piece of snake food at a random location.
void Snake::genNewFood() {
  do {
    foodPos = random(NUM_PIXELS);
  } while(!isOpen(foodPos) && foodPos != snakePos);

  history[0] = foodPos;
}

// Determines what each button means in terms of game control.
bool Snake::parseIR (uint32_t recvCode)
{
  switch(recvCode)
  {
     case BUTTON_LEFT:
        if (direction != dirRIGHT)
          direction=dirLEFT;
        break;
     case BUTTON_RIGHT:
        if (direction != dirLEFT)
          direction=dirRIGHT;
        break;
     case BUTTON_UP:
        if (direction != dirDOWN)
          direction=dirUP;
        break;
     case BUTTON_DOWN:
        if (direction != dirUP)
          direction=dirDOWN;
        break;
     default:
        return false;
  }
  updatePos();
  count = 0;
  return true;
}

// Accessor functions.
uint8_t Snake::getNumItems()
{
  return numItems+1;
}

void Snake::newGame()
{
  // Cleaning out the history from the previous game.
  for (int i = 0; i <= numItems; i++)
    history[i]=0;
  // Start in the center of the map.
  posx = BOARD_HEIGHT/2;
  posy = BOARD_WIDTH/2;
  count = 0;
  direction = 0;
  // Add one for food tile.
  numItems = SNAKE_START_LENGTH + 1;
  snakePos = BOARD_WIDTH*posx + posy;
  history[1] = snakePos;

  genNewFood();
}

// Primary function that runs game mechanics.
void Snake::playGame()
{
  count++;
  if (count >= SNAKE_SPEED) {
    updatePos();
    count = 0;
  }
}

// Updates the snake's position in memory.
void Snake::updatePos () {

  // Updates the direction when given a new code.
  switch(direction) {
    case dirUP:
        posx--;
        break;
      case dirDOWN:
        posx++;
        break;
      case dirRIGHT:
        posy--;
        break;
      case dirLEFT:
        posy++;
        break;
      default:
        break;
  }

  snakePos = BOARD_WIDTH*posx + posy;

  for (int i = numItems; i > 0; i--) {
    if (i == 1)
      history[i] = snakePos;
    else
      history[i] = history[i-1];
  }

  // Checks for collision with food.
  if (snakePos == foodPos)
  {
    numItems++;
    history[numItems] = history[numItems-1];
    genNewFood();
  }

  if (!isOpen(snakePos))
    newGame();

  if (!(posx >= 0 && posx <= BOARD_HEIGHT-1 && posy >= 0 && posy <= BOARD_WIDTH-1))
    newGame();
}
